@@ -1,10 +1,9 @@
 import numpy
 import numpy.random
-from .blis import gemm_
-from .blis import init
+from .py import gemm, einsum
 from timeit import default_timer as timer
 numpy.random.seed(0)
-init()
+
 
 def create_data(nO, nI, batch_size):
     X = numpy.zeros((batch_size, nI), dtype='f')
@@ -14,56 +13,92 @@ def create_data(nO, nI, batch_size):
     return X, W
 
 
-def run_numpy(X, W):
-    nO, nI = W.shape
-    batch_size = X.shape[0]
-    total = 0.
-    for i in range(1000):
-        y = numpy.dot(X, W)
-        total += y.sum()
-    print(total)
+def get_numpy_blas():
+    blas_libs = numpy.__config__.blas_opt_info['libraries']
+    return blas_libs[0]
 
 
-def run_blis(X, W):
+def numpy_gemm(X, W, n=1000):
     nO, nI = W.shape
     batch_size = X.shape[0]
     total = 0.
     y = numpy.zeros((batch_size, nO), dtype='f')
-    for i in range(1000):
-        # Output dim: batch_size * nr_row
-        # vec dim:    batch_size * nr_col
-        # mat dim:    nr_row     * nr_col
-        # vec:   M * K
-        # mat.T: K * N
-        # out: M * N
-        gemm_(0, 0, batch_size, nO, nI,
-              1.0,
-              X, X.shape[1], 1,
-              W, W.shape[1], 1,
-              1.0,
-              y, y.shape[1], 1)
+    for i in range(n):
+        numpy.dot(X, W, out=y)
+        total += y.sum()
+        y.fill(0)
+    print('Total:', total)
+
+
+def blis_gemm(X, W, n=1000):
+    nO, nI = W.shape
+    batch_size = X.shape[0]
+    total = 0.
+    y = numpy.zeros((batch_size, nO), dtype='f')
+    for i in range(n):
+        gemm(X, W, out=y)
         total += y.sum()
         y.fill(0.)
-    print(total)
+    print('Total:', total)
+
+
+def numpy_einsum(X, W, n=1000):
+    nO, nI = W.shape
+    batch_size = X.shape[0]
+    total = 0.
+    y = numpy.zeros((nO, batch_size), dtype='f')
+    for i in range(n):
+        numpy.einsum('ab,cb->ca', X, W, out=y)
+        total += y.sum()
+        y.fill(0.)
+    print('Total:', total)
+
+
+def blis_einsum(X, W, n=1000):
+    nO, nI = W.shape
+    batch_size = X.shape[0]
+    total = 0.
+    y = numpy.zeros((nO, batch_size), dtype='f')
+    for i in range(n):
+        einsum('ab,cb->ca', X, W, out=y)
+        total += y.sum()
+        y.fill(0.)
+    print('Total:', total)
+
 
 def main(nI=128*3, nO=128*3, batch_size=2000):
-    print("Setting up data "
+    print("Setting up data for gemm. 1000 iters,  "
           "nO={nO} nI={nI} batch_size={batch_size}".format(**locals()))
+    numpy_blas = get_numpy_blas()
     X1, W1 = create_data(nI, nO, batch_size)
     X2 = X1.copy()
     W2 = W1.copy()
-    print("Blis...")
+    print("Blis gemm...")
     start = timer()
-    run_blis(X2, W2)
+    blis_gemm(X2, W2, n=1000)
     end = timer()
     blis_time = end-start
     print("%.2f seconds" % blis_time) 
-    print("Numpy...")
+    print("Numpy (%s) gemm..." % numpy_blas)
     start = timer()
-    run_numpy(X1, W1)
+    numpy_gemm(X1, W1)
+    end = timer()
+    numpy_time = end-start
+    print("%.2f seconds" % numpy_time) 
+    print("Blis einsum ab,cb->ca")
+    start = timer()
+    blis_einsum(X2, W2, n=1000)
+    end = timer()
+    blis_time = end-start
+    print("%.2f seconds" % blis_time) 
+    print("Numpy (%s) einsum ab,cb->ca" % numpy_blas)
+    start = timer()
+    numpy_einsum(X2, W2)
     end = timer()
     numpy_time = end-start
     print("%.2f seconds" % numpy_time) 
 
+
+
 if __name__:
     main()
